//*****************************************
// AVR ATmega644P Geiger Counter
// CPU Clock:8MHz,16NHz,18.432MHz or 20MHz(Compile Option)
// File Name:Comm1.c
//  Use UART0
// KCK[JE^[p
//   iq[Yf[^@is`fgpȂAȂAwIV[^(8MHz)Aɂj
//   i`FbN͂rohdmj
//
//  C:\WinAVR-20100110\doc\avr-libc\avr-libc-user-manual.pdf ɊeCN[ht@C̐
//
//  2011/09/02 New  Taishi ATmega644P(J308)
//  2011/10/22 Modify for LND712 by  Taishi
//  2011/10/28 Modify for SBM-20 by  Taishi
//  2011/11/08 Modify for LND712 by  Taishi
//  2011/11/12 Modify for SBM-20 by  Taishi
//  2011/11/12 Modify for J-308  by  Taishi
//  2011/11/24 Modify for All Model  Taishi
//  2012/03/28 Modify for All Model  Taishi(Ver 1.11) 
//*****************************************
#include				<avr/interrupt.h>
#include				<avr/io.h>
#include				<avr/pgmspace.h>

#include				"AVR_ATmega644_Geiger.h"
#include				"Serial.h"
#include				"LCD.h"
#include				"Utility.h"
#include				"Setting.h"
#include				"Display.h"

#include				"Comm1.h"


// peB@t`qsO
#define	V_PARITY0_NON	((0 << UPM01) | (0 << UPM00))									// peB
#define	V_PARITY0_EVN	((1 << UPM01) | (0 << UPM00))									// peB
#define	V_PARITY0_ODD	((1 << UPM01) | (1 << UPM00))									// peB

// f[^@t`qsO
#define	V_DATA0_8		((1 << UCSZ00) | (1 << UCSZ01))									// Wrbgf[^
#define	V_DATA0_7		((0 << UCSZ00) | (1 << UCSZ01))									// Vrbgf[^

// Xgbvrbg@t`qsO
#define	V_STOP0_1		(0 << USBS0)													// PrbgXgbv
#define	V_STOP0_2		(1 << USBS0)													// QrbgXgbv

// tbrqOa̒l
#ifdef		DEBUG_TX
#define	V_UCSR0B		((1 << RXEN0) | (1 << RXCIE0) | (1 << TXEN0))					// tbrqOa̒l: MAM荞݋AM
#else		//DEBUG_TX
#define	V_UCSR0B		((1 << RXEN0) | (1 << RXCIE0))									// tbrqOa̒l: MAM荞݋
#endif		//DEBUG_TX


unsigned int	v_parity0[CODE_PARITY_MAX+1] ={				// peBɑ΂ enum code_parity Ɠт̕KvL
						V_PARITY0_NON,						// peB
						V_PARITY0_EVN,						// peB
						V_PARITY0_ODD						// peB
				};

unsigned int	v_data0_length[CODE_DATA_MAX+1] ={			// f[^ɑ΂ enum code_data_length Ɠт̕KvL
						V_DATA0_8,							// Wrbgf[^
						V_DATA0_7							// Vrbgf[^
				};



volatile unsigned int	uart0_err_receiver;					// qwOG[ƎMf[^

#define			NofRxdBuf				16					// Mf[^obt@̑傫iQ̊KɌj
#define			DATA_WARITE_POS_MASK	(NofRxdBuf - 1)
unsigned char	rxd_data[NofRxdBuf];						// Mf[^obt@
unsigned char	write_rxd_pos;
unsigned char	read_rxd_pos;


//***************************
// UART0 Disable
//***************************
void	uart0_disable(void)
{
	UCSR0B = 0;
}
//***************************
// UART0 Pre Initial
//***************************
void	uart0_pre_init(void)
{
#ifdef	UART_BAISOKU													// t`qs̃{[[gpNbN{ݒɂ
	UCSR0A |= (1 << U2X0);												// {
#endif

	UBRR0  = v_ubrr[set_bps];											// aor

	UCSR0C = (v_parity0[set_parity] | v_data0_length[set_data_length]);	// peBAf[^

//	UCSR0B = ((1 << TXEN0) | (1 << RXEN0));								// tbrqOa̒l: MƎM
	UCSR0B = (1 << TXEN0);												// tbrqOa̒l: M

}
//***************************
// UART0 Initial
// CPU Clock:8MHz,16NHz,18.432MHz or 20MHz(Compile Option)
//***************************
void	uart0_init(void)
{
#ifdef	UART_BAISOKU													// t`qs̃{[[gpNbN{ݒɂ
	UCSR0A |= (1 << U2X0);												// {
#endif

	UBRR0  = v_ubrr[set_bps];											// aor

	UCSR0C = (v_parity0[set_parity] | v_data0_length[set_data_length]);	// peBAf[^

	UCSR0B = V_UCSR0B;

}
//***************************
// UART0 Initial for GPS
// CPU Clock:8MHz,16NHz,18.432MHz or 20MHz(Compile Option)
//***************************
void	uart0_gps_init(void)
{
	unsigned char	i;
#ifdef	UART_BAISOKU													// t`qs̃{[[gpNbN{ݒɂ
	UCSR0A |= (1 << U2X0);												// {
#endif

	UBRR0  = v_ubrr[set_bps];											// aor

	UCSR0C = (v_parity0[set_parity] | v_data0_length[set_data_length]);	// peBAf[^

	UCSR0B = ((1 << RXEN0) | (1 << RXCIE0) | (1 << TXEN0));

	for(i=0;i<NofRxdBuf;i++){
		rxd_data[i] = 0;
	}
	write_rxd_pos = 0;
	read_rxd_pos = 0;
}

//***************************
// UART0 RXD Interrupt
//***************************
ISR	(USART0_RX_vect)
{
//	uart0_err_receiver = (UCSR0A & ~0x01);										// G[Ԏ擾iŉʃrbgOFqwOj
//	uart0_err_receiver <<= 8;
//	uart0_err_receiver |= UDR0;													// Mf[^擾

	unsigned char	er;
	unsigned char	uc;
	unsigned char	wr_pos;

	er = (UCSR0A & ((1<<FE0)|(1<<DOR0)|(1<<UPE0)) );	// G[Ԏ擾
	uc = UDR0;											// Mf[^擾

	if(er == 0){										// G[
		wr_pos = write_rxd_pos;
		rxd_data[wr_pos] = uc;							// Mf[^obt@Ɏ擾
		wr_pos++;
		wr_pos &= DATA_WARITE_POS_MASK;
		if(wr_pos != read_rxd_pos){						// ݈ʒuǂݏoʒuɒǂĂȂ
			write_rxd_pos = wr_pos;
		}
	}
}
//***************************
// RXD Read Call from Main
//   uc: Mf[^鏊
// ߂l  : 0=Mf[^
//         : 1=Mf[^L
//***************************
unsigned char	Get_rxd(unsigned char *uc)
{
	unsigned char	r=0;
	if(write_rxd_pos == read_rxd_pos){						// Mf[^
		;
	}else{
		r = 1;
		*uc = rxd_data[read_rxd_pos];						// Mf[^obt@擾
		read_rxd_pos++;
		read_rxd_pos &= DATA_WARITE_POS_MASK;
	}
	return(r);
}
//***************************
// UART0 RXD One Data
//  荞݂gȂ@PoCgM
//  ߂lFOMȂ
//  ߂lFPMAMf[^ uart0_err_receiver ɓ
//***************************
unsigned char	rx0_char(void)
{
	unsigned char	r = 0;

	if((UCSR0A & (1 << RXC0)) != 0){
		uart0_err_receiver = (UCSR0A & ~(FE0 | DOR0 | UPE0));					// G[Ԏ擾
		uart0_err_receiver <<= 8;
		uart0_err_receiver |= UDR0;												// Mf[^擾
		r = 1;
	}
	return(r);
}
//***************************
// UART0 TXD One Data
//***************************
void	tx0_char(unsigned char uc)
{
	while((UCSR0A & (1 << UDRE0)) == 0){
		;
	}
	UDR0 = uc;								// Mf[^
}
//***************************
// UART0 TXD String
//***************************
void	tx0_str(unsigned char *uc)
{
	while(*uc != 0){
		tx0_char(*uc);						// UART0 TXD One Data
		uc++;
	}
}
//***************************
// UART0 TxD String + CR + LF
//***************************
void	uart0_tx_str_crlf(char *str)
{
	tx0_str((unsigned char *)str);							// UART0 TxD String
	tx0_char(0x0d);							// CR
	tx0_char(0x0a);							// LF
}


unsigned char	tx0_moji[4];													// fobOʐMp
//*****************************************
// fobOp\
// KX@e͕ύX
// tx_debug
//*****************************************
void	tx_debug(unsigned int num, unsigned char uc)
{
	wtod_03u((char *)&tx0_moji[0], num);
	tx0_str(tx0_moji);
	tx0_str((unsigned char *)":");
	wtod_02x((char *)&tx0_moji[0], uc);
	tx0_str(tx0_moji);
	tx0_str((unsigned char *)"H\r\n");
}
//*****************************************
// fobOp\
// KX@e͕ύX
// tx_debug
//*****************************************
void	tx_debug2(unsigned int num1, unsigned int num2, unsigned char uc)
{
	wtod_03u((char *)&tx0_moji[0], num1);
	tx0_str(tx0_moji);
	tx0_str((unsigned char *)",");
	wtod_02x((char *)&tx0_moji[0], uc);
	wtod_03u((char *)&tx0_moji[0], num2);
	tx0_str(tx0_moji);
	tx0_str((unsigned char *)":");
	wtod_02x((char *)&tx0_moji[0], uc);
	tx0_str(tx0_moji);
	tx0_str((unsigned char *)"H\r\n");
}
//*****************************************
// fobOpRPOi\
// KX@e͕ύX
// tx_debug
//*****************************************
void	tx_03_debug(unsigned int n)
{
	wtod_03u((char *)&tx0_moji[0], n);
	tx0_str(tx0_moji);
	tx0_str((unsigned char *)"\r\n");
}
